import datetime

from botocore.exceptions import ClientError
from jinja2 import Environment, FileSystemLoader
import os
from pyppeteer import launch

from src.widgets import create_daily_report_widgets
from src.processor import daily_data_processor
import boto3
import logging
import json

logger = logging.getLogger()


async def daily_report_generator(data):
    HERE = os.path.dirname(os.path.abspath(__file__))
    J2 = Environment(loader=FileSystemLoader(HERE), trim_blocks=True)
    formatted_data = {}
    logger.info("Getting the numerical values in the daily report")
    formatted_data['likely_threats'] = daily_data_processor.get_num_likely_threats(data)
    formatted_data[
        'average_time_to_determination_likely_alerts'] = daily_data_processor.get_avg_time_to_determination_likely_threats(
        data)
    formatted_data['eligible_for_mdr_alerts'] = daily_data_processor.get_num_mdr_eligible_alerts(data)
    formatted_data[
        'average_time_to_determination_eligible_alerts'] = daily_data_processor.get_avg_time_to_determination_mdr_eligible_alerts(
        data)
    dates_list = daily_data_processor.get_start_and_end_date()
    formatted_data['start_time_frame'] = dates_list[0]
    formatted_data['end_time_frame'] = dates_list[1]
    formatted_data['date_range'] = daily_data_processor.get_time_range()

    logger.info("Getting the plots in the daily report")

    formatted_data['pie_likely_severity'] = create_daily_report_widgets.plot_pie_severity(
        daily_data_processor.get_pie_severity_scores(data, 'likely'), 'likely')
    formatted_data['pie_mdr_severity'] = create_daily_report_widgets.plot_pie_severity(
        daily_data_processor.get_pie_severity_scores(data, 'mdr'), 'mdr')

    formatted_data['line_likely_count'] = create_daily_report_widgets.plot_line_chart(
        daily_data_processor.get_line_chart(data, 'likely_count'), 'likely', 'count')
    formatted_data['line_likely_avg_time'] = create_daily_report_widgets.plot_line_chart(
        daily_data_processor.get_line_chart(data, 'likely_avg_ttr'), 'likely', 'avg')

    formatted_data['line_mdr_count'] = create_daily_report_widgets.plot_line_chart(
        daily_data_processor.get_line_chart(data, 'mdr_count'), 'mdr', 'count')
    formatted_data['line_mdr_avg_time'] = create_daily_report_widgets.plot_line_chart(
        daily_data_processor.get_line_chart(data, 'mdr_avg_ttr'), 'mdr', 'avg')

    formatted_data['bar_likely_severity'] = create_daily_report_widgets.plot_bar_severity(
        daily_data_processor.get_bar_chart_severity(data, 'likely'), 'likely')
    formatted_data['bar_mdr_severity'] = create_daily_report_widgets.plot_bar_severity(
        daily_data_processor.get_bar_chart_severity(data, 'mdr'), 'mdr')

    formatted_data['bar_horizontal_process_likely'] = create_daily_report_widgets.plot_barh_process(
        daily_data_processor.get_list_hchart(data, 'likely', 'app'), 'likely')
    formatted_data['bar_horizontal_asset_likely'] = create_daily_report_widgets.plot_barh_asset(
        daily_data_processor.get_list_hchart(data, 'likely', 'dev'), 'likely')

    formatted_data['bar_horizontal_process_mdr'] = create_daily_report_widgets.plot_barh_process(
        daily_data_processor.get_list_hchart(data, 'mdr', 'app'), 'mdr')
    formatted_data['bar_horizontal_asset_mdr'] = create_daily_report_widgets.plot_barh_asset(
        daily_data_processor.get_list_hchart(data, 'mdr', 'dev'), 'mdr')

    template = J2.get_template('templates/dailyReport/dailyReport.html')
    html = template.render(formatted_data)
    file_html = open("daily_report_generated.html", "w")
    # Adding the input data to the HTML file
    file_html.write(html)
    file_html.close()
    await generate_pdf_from_html(f'{data.get("org")}_daily_report.pdf')


async def generate_pdf_from_html(pdf_path):
    browser = await launch(
        headless=True,
        executablePath='/usr/lib/chromium-browser/headless_shell',
        args=[
            '--no-sandbox',
            '--single-process',
            '--disable-dev-shm-usage',
            '--disable-gpu',
            '--no-zygote'
        ]
    )
    page = await browser.newPage()
    logger.info(f'file://{os.getcwd()}/daily_report_generated.html')

    await page.goto(f'file://{os.getcwd()}/daily_report_generated.html', waitUntil='networkidle0')

    logger.info('set content finished')

    await page.pdf({'path': pdf_path, 'format': 'A3', 'margin': {'top': '40'}})

    await browser.close()


def from_s3(key, bucket='kcdev01-mdr-reporting-job-infra-daily-alert', endpoint='https://s3.us-east-1.amazonaws.com'):
    logger.info(f'Retrieving {key} from {bucket} from {endpoint}')
    s3_client = boto3.client('s3', endpoint_url=endpoint)
    try:
        obj = s3_client.get_object(Bucket=bucket, Key=key)
        body = obj.get('Body').read().decode('utf-8')
        return json.loads(body)
    except ClientError as exc:
        logger.exception(f'failed to retrieve {bucket}/{key} from {endpoint}: {exc}')
        raise exc


def put_to_s3(filepath, key, bucket='kcdev01-mdr-reporting-generator-infra-report-pdf',
              endpoint='https://s3.us-east-1.amazonaws.com'):
    try:
        logger.info(f'Uploading {filepath} to {bucket}/{key}')
        s3 = boto3.resource('s3', endpoint_url=endpoint)
        key = get_daily_report_key(key)
        obj = s3.Object(bucket, key)
        obj.upload_file(Filename=filepath)
    except ClientError as exc:
        logger.exception(f'failed to upload {bucket}/{key} to {endpoint}: {exc}')
        raise exc


def get_daily_report_key(key):
    dates_list = daily_data_processor.get_start_and_end_date()
    return f'daily_report/{key}/{datetime.datetime.strptime(dates_list[0], "%B %d, %Y %I:%M%p UTC").strftime("%Y-%m-%d")}.pdf'
