import json

import boto3
from botocore.exceptions import ClientError
from jinja2 import Environment, FileSystemLoader
import os
import logging
from pyppeteer import launch
from src.widgets import create_monthly_report_widgets
from src.processor import monthly_data_processor

logger = logging.getLogger()


async def monthly_report_generator(data):
    HERE = os.path.dirname(os.path.abspath(__file__))
    J2 = Environment(loader=FileSystemLoader(HERE), trim_blocks=True)
    formatted_data = {}
    logger.info("Getting the numerical values in the monthly report")

    formatted_data['likely_threats'] = monthly_data_processor.get_num_threats(data, 'likely')
    formatted_data[
        'average_time_to_determination_likely_alerts'] = monthly_data_processor.get_avg_time_to_determination(data,
                                                                                                              'likely')
    formatted_data['eligible_for_mdr_alerts'] = monthly_data_processor.get_num_threats(data, 'mdr')
    formatted_data[
        'average_time_to_determination_eligible_alerts'] = monthly_data_processor.get_avg_time_to_determination(data,
                                                                                                                'mdr')
    start_end = monthly_data_processor.get_start_and_end_date()
    formatted_data['start_time_frame'] = start_end[0]
    formatted_data['end_time_frame'] = start_end[1]
    formatted_data['date_range'] = monthly_data_processor.get_plot_date_range()
    formatted_data['mdr_eligible_alerts'] = monthly_data_processor.get_num_threats(data, 'mdr')
    formatted_data[
        'average_time_to_determination_mdr_eligible_alerts'] = monthly_data_processor.get_avg_time_to_determination(
        data, 'mdr')
    formatted_data['last_month_date_range'] = monthly_data_processor.get_plot_month_range()
    fixed_data = monthly_data_processor.get_fixed_values(data)
    formatted_data['mdr_quarantine_action'] = fixed_data[0]
    formatted_data['mdr_policy_action'] = fixed_data[1]
    formatted_data['alert_notification_subscription_recipient'] = fixed_data[2]
    formatted_data['daily_report_subscription_recipient'] = fixed_data[3]
    formatted_data['monthly_report_subscription_recipient'] = fixed_data[4]

    # Industry comparison
    formatted_data['industry'] = fixed_data[5]
    formatted_data['industry_size'] = fixed_data[6]
    formatted_data['customer_average'] = fixed_data[7]
    formatted_data['size_average'] = fixed_data[8]
    formatted_data['industry_average'] = fixed_data[9]
    logger.info("Getting the plots in the monthly report")

    formatted_data['line_likely_count_monthly'] = create_monthly_report_widgets.create_line_plot_monthly(
        monthly_data_processor.get_line_plot_data(data, 'likely_count'), 'likely', 'count')
    formatted_data['line_likely_avg_monthly'] = create_monthly_report_widgets.create_line_plot_monthly(
        monthly_data_processor.get_line_plot_data(data, 'likely_avg_ttr'), 'likely', 'avg')
    formatted_data['line_likely_count_half_yearly'] = create_monthly_report_widgets.create_line_plots()
    formatted_data['line_likely_avg_half_yearly'] = create_monthly_report_widgets.create_line_plots()
    formatted_data['pie_likely_severity_monthly'] = create_monthly_report_widgets.create_pie_3_chart(
        monthly_data_processor.get_pie_severity_scores(data, 'likely'), 'likely')
    formatted_data['bar_likely_severity_half_yearly'] = create_monthly_report_widgets.create_barh_4_plot()
    formatted_data['bar_horizontal_process_likely_monthly'] = create_monthly_report_widgets.create_barh_process(
        monthly_data_processor.get_list_hplot(data, 'likely', 'app'), 'likely')
    formatted_data['bar_horizontal_asset_likely_monthly'] = create_monthly_report_widgets.create_barh_asset(
        monthly_data_processor.get_list_hplot(data, 'likely', 'dev'), 'likely')
    formatted_data['area_chart_monthly'] = create_monthly_report_widgets.create_area_plot()
    formatted_data['area_chart_half_yearly'] = create_monthly_report_widgets.create_area_plot()
    formatted_data['multi_line_chart_half_yearly'] = create_monthly_report_widgets.create_multi_line_plots()

    formatted_data['line_mdr_count_monthly'] = create_monthly_report_widgets.create_line_plot_monthly(
        monthly_data_processor.get_line_plot_data(data, 'mdr_count'), 'mdr', 'count')
    formatted_data['line_mdr_avg_monthly'] = create_monthly_report_widgets.create_line_plot_monthly(
        monthly_data_processor.get_line_plot_data(data, 'mdr_avg_ttr'), 'mdr', 'avg')
    formatted_data['line_mdr_count_half_yearly'] = create_monthly_report_widgets.create_line_plots()
    formatted_data['line_mdr_avg_half_yearly'] = create_monthly_report_widgets.create_line_plots()
    formatted_data['pie_mdr_severity_monthly'] = create_monthly_report_widgets.create_pie_3_chart(
        monthly_data_processor.get_pie_severity_scores(data, 'mdr'), 'mdr')
    formatted_data['bar_mdr_severity_half_yearly'] = create_monthly_report_widgets.create_barh_4_plot()
    formatted_data['bar_horizontal_process_mdr_monthly'] = create_monthly_report_widgets.create_barh_process(
        monthly_data_processor.get_list_hplot(data, 'mdr', 'app'), 'mdr')
    formatted_data['bar_horizontal_asset_mdr_monthly'] = create_monthly_report_widgets.create_barh_asset(
        monthly_data_processor.get_list_hplot(data, 'mdr', 'dev'), 'mdr')
    template = J2.get_template('templates/monthlyReport/monthlyReport.html')
    html = template.render(formatted_data)
    file_html = open("monthly_report_generated.html", "w")
    # Adding the input data to the HTML file
    file_html.write(html)
    file_html.close()
    await generate_pdf_from_html(f'{data.get("org")}_monthly_report.pdf')


async def generate_pdf_from_html(pdf_path):
    browser = await launch(
        headless=True,
        executablePath='/usr/lib/chromium-browser/headless_shell',
        args=[
            '--no-sandbox',
            '--single-process',
            '--disable-dev-shm-usage',
            '--disable-gpu',
            '--no-zygote'
        ]
    )
    page = await browser.newPage()
    logger.info(f'file://{os.getcwd()}/monthly_report_generated.html')

    await page.goto(f'file://{os.getcwd()}/monthly_report_generated.html', waitUntil='networkidle0')

    logger.info('set content finished')

    await page.pdf({'path': pdf_path})

    await browser.close()


def from_s3(key, bucket='kcdev01-mdr-reporting-job-infra-monthly-alert', endpoint='https://s3.us-east-1.amazonaws.com'):
    logger.info(f'Retrieving {key} from {bucket} from {endpoint}')
    s3_client = boto3.client('s3', endpoint_url=endpoint)
    try:
        obj = s3_client.get_object(Bucket=bucket, Key=key)
        body = obj.get('Body').read().decode('utf-8')
        return json.loads(body)
    except ClientError as exc:
        logger.exception(f'failed to retrieve {bucket}/{key} from {endpoint}: {exc}')
        raise exc


def put_to_s3(filepath, key, bucket='kcdev01-mdr-reporting-generator-infra-report-pdf',
              endpoint='https://s3.us-east-1.amazonaws.com'):
    try:
        logger.info(f'Uploading {filepath} to {bucket}/{key}')
        s3 = boto3.resource('s3', endpoint_url=endpoint)
        key = get_monthly_report_key(key)
        obj = s3.Object(bucket, key)
        obj.upload_file(Filename=filepath)
    except ClientError as exc:
        logger.exception(f'failed to upload {bucket}/{key} to {endpoint}: {exc}')
        raise exc


def get_monthly_report_key(key):
    report_year, report_month = monthly_data_processor.get_report_year_month()
    return f'monthly_report/{key}/{report_year}-{report_month}.pdf'
