import logging
import os
import sys
import datetime
from calendar import monthrange

from dateutil.relativedelta import relativedelta

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)

AWS_REGION = os.getenv("AWS_REGION", "us-east-1")


# Checking if date is in a given range
def is_date_in_range(date_check):
    report_year, report_month = get_report_year_month()
    start = datetime.datetime(day=1, month=report_month, year=report_year)
    end = datetime.datetime(day=monthrange(report_year, report_month)[1], month=report_month, year=report_year)
    if start <= datetime.datetime.strptime(date_check, "%Y-%m-%d") <= end:
        return True
    else:
        return False


# Get month and year of the monthly report
def get_report_year_month():
    today = datetime.date.today()
    first = today.replace(day=1)
    prev_date = first - datetime.timedelta(days=1)
    return [prev_date.year, prev_date.month]


# Get number of threats
def get_num_threats(data, alert_type):
    total_threats = 0
    counter = 0  # To differentiate between data being empty vs total threats being 0
    for dct in data["statistics"]:
        counter += 1
        if is_date_in_range(dct['create_date']):
            total_threats += int(dct[alert_type + '_count'])
    if counter > 0:
        return total_threats
    else:
        return '--'


# Get avg time to determination
def get_avg_time_to_determination(data, alert_type):
    total_avg_det_time = 0
    counter = 0
    for dct in data["statistics"]:
        if is_date_in_range(dct['create_date']):
            counter += 1
            total_avg_det_time += float(dct[alert_type + '_avg_ttr'])
    if counter != 0:
        return round(total_avg_det_time / counter, 1)
    else:
        return '--'

# Getting start and date in the report title
def get_start_and_end_date():
    report_year, report_month = get_report_year_month()
    start_date = datetime.date(report_year, report_month, 1)
    end_date = datetime.date(report_year, report_month, monthrange(report_year, report_month)[1])
    start_date = start_date.strftime('%B %d, %Y 2:00am UTC')
    end_date = end_date.strftime('%B %d, %Y 2:00am UTC')
    dates_list = [start_date, end_date]
    return dates_list

# Getting date range in the plots
def get_plot_date_range():
    report_year, report_month = get_report_year_month()
    end_date = datetime.date(report_year, report_month, monthrange(report_year, report_month)[1])
    month = end_date.strftime('%b')
    return month + ' 1-' + end_date.strftime('%d')

# Getting month range in the plots
def get_plot_month_range():
    report_year, report_month = get_report_year_month()
    end_date = datetime.date(report_year, report_month, monthrange(report_year, report_month)[1])
    start_date = end_date - relativedelta(months=-7)
    year = end_date.strftime('%Y')
    end_month = end_date.strftime('%B')
    start_month = start_date.strftime('%B')
    return start_month + '-' + end_month + ', ' + year

# Getting numerical data in the monthly report plots
def get_fixed_values(data):
    if data['statistics'] and data['statistics'][0]:
        return [data['statistics'][0]['mdr_quarantine'], data['statistics'][0]['mdr_policy'],
                data['statistics'][0]['alerts_enabled_count'],
                data['statistics'][0]['daily_summary_enabled_count'],
                data['statistics'][0]['monthly_report_enabled_count'],
                data['statistics'][0]['account_industry'], data['statistics'][0]['org_size'],
                round(float(data['statistics'][0]['alert_per_asset']), 1),
                round(float(data['statistics'][0]['size_alert_per_asset']), 1),
                round(float(data['statistics'][0]['industry_alerts_per_asset']), 1)]
    else:
        return ['--'] * 10

# Getting data for the line plots
def get_line_plot_data(data, yValue):
    temp_lst = []
    for dct in data["statistics"]:
        l = [datetime.datetime.strptime(dct["create_date"], "%Y-%m-%d").date().strftime("%b %-d %Y"), dct[yValue]]
        temp_lst.append(l)

    temp_lst.sort(key=lambda date: datetime.datetime.strptime(date[0], "%b %d %Y"))
    dates = [x[0] for x in temp_lst]
    if not dates:
        return [[], []]
    if '_avg_ttr' in yValue:
        y = [round(float(x[1]), 1) for x in temp_lst]
    else:
        y = [int(x[1]) for x in temp_lst]
    final_lst = []
    counter = 0
    report_year, report_month = get_report_year_month()
    for x in range(1, monthrange(report_year, report_month)[1] + 1):
        if x < int(datetime.datetime.strptime(dates[counter], "%b %d %Y").date().strftime("%d")):
            final_lst.append(0)
        else:
            final_lst.append(y[counter])
            counter += 1
    start_end_date = [datetime.date(report_year, report_month, 1).strftime("%b %d %Y"),
                      datetime.date(report_year, report_month, monthrange(report_year, report_month)[1]).strftime(
                          "%b %d %Y")]
    return [start_end_date, final_lst]

# Getting data for the horizontal bar plots
def get_list_hplot(data, alertType, dataType):
    tempdict = {}
    for k in range(1, 7):
        for dct in data["statistics"]:
            key = alertType + '_' + dataType + '_' + str(k)
            if not (dct[key] == "" or not dct[key]) and not (dct[key + '_cnt'] == "" or not dct[key + '_cnt']):
                if dct[key] in tempdict:
                    tempdict[dct[key]] += int(dct[key + '_cnt'])
                else:
                    tempdict[dct[key]] = int(dct[key + '_cnt'])
    if not tempdict:
        return [[], []]
    else:
        return [list(tempdict.keys()), list(tempdict.values())]

# Getting data for the pie chart
def get_pie_severity_scores(data, alertType):
    tempdict = {}
    for k in range(5, 11):
        for dct in data["statistics"]:
            key = alertType + '_alert_' + str(k)
            if dct[key] == '':
                dct[key] = 0
            if k in tempdict:
                tempdict[k] += int(dct[key])
            else:
                tempdict[k] = int(dct[key])
    return tempdict
