import json
import logging
import os
import sys
import time

import boto3

from src import daily_report_generator, monthly_report_generator
import asyncio

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)

AWS_REGION = os.getenv("AWS_REGION", "us-east-1")


def get_alert_data_key(message_body):
    # Parse the message body as JSON
    message = json.loads(message_body)

    # The structure of the message is such that the object key
    # is located within 'Records', then within 's3' and 'object'
    if 'Records' in message and len(message['Records']) > 0:
        s3_event = message['Records'][0]
        if 's3' in s3_event and 'object' in s3_event['s3']:
            bucket_name = s3_event['s3']['bucket']['name']
            object_key = s3_event['s3']['object']['key']
            return [bucket_name,object_key]
            # Add your processing logic here
        else:
            print("Could not find 's3' or 'object' in the message.")
            # TODO: raise exception
            return None
    else:
        print("No 'Records' found in the message.")
        return None


async def main():
    try:
        # Initialize the SQS client
        sqs = boto3.client('sqs', region_name=AWS_REGION)

        # Specify your queue URL here
        queue_url = os.getenv("ALERT_BUCKET_EVENT_SQS_URL", None)

        while True:
            # Receive message from SQS queue
            response = sqs.receive_message(
                QueueUrl=queue_url,
                MaxNumberOfMessages=1,
                WaitTimeSeconds=20  # Long polling
            )
            logger.info(f"Processing new message from sqs")
            logger.info(response)
            if 'Messages' not in response:
                continue
            # Get the first message
            logger.info(f"Starting the processing")
            message = response['Messages'][0]
            receipt_handle = message['ReceiptHandle']

            # Process the message
            bucket_name, alert_obj_key = get_alert_data_key(message['Body'])
            logger.info(f"Processing {alert_obj_key} in bucket {bucket_name}")
            if 'daily' in bucket_name:
                logger.info(f"Getting daily raw data from S3 bucket {bucket_name}")
                daily_raw_data = daily_report_generator.from_s3(alert_obj_key)
                logger.info("Processing raw data and creating daily report html")
                await daily_report_generator.daily_report_generator(daily_raw_data)
                logger.info("Converted daily report html to pdf")
                logger.info("Pushing daily report pdf to generator bucket")
                daily_report_generator.put_to_s3(f'{daily_raw_data.get("org")}_daily_report.pdf', daily_raw_data.get("org"))
            if 'monthly' in bucket_name:
                logger.info(f"Getting monthly raw data from S3 bucket {bucket_name}")
                monthly_raw_data = monthly_report_generator.from_s3(alert_obj_key)
                logger.info("Processing raw data and creating monthly report html")
                await monthly_report_generator.monthly_report_generator(monthly_raw_data)
                logger.info("Converted monthly report html to pdf")
                logger.info("Pushing monthly report pdf to generator bucket")
                monthly_report_generator.put_to_s3(f'{monthly_raw_data.get("org")}_monthly_report.pdf', monthly_raw_data.get("org"))

            # TODO: Check and/or update postgres report status
            # TODO: Convert org alert data to HTML then convert it to PDF
            # TODO: Put PDF file to s3 bucket

            # Delete received message from queue
            sqs.delete_message(
                QueueUrl=queue_url,
                ReceiptHandle=receipt_handle
            )
    except:
        logger.exception("Exception while generating reports")
        time.sleep(15)
if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())

